-- Main Driver

do
	
	-- Utilities
	u = require('lib.utils')
	
	-- Driver Global
	_D = {
		debug = false
	}
	
	-- MQTT Module
	MQTT = require('lib.mqtt')
	
end

-- Properties

function OnPropertyChanged(strProperty)

	u.debug("OnPropertyChanged (" .. strProperty .. ")")

	-- Debug Mode
	if strProperty == "Debug Mode" then

		_D.debug = Properties[strProperty] == "On"
	
		u.console("Debug Mode Turned " .. Properties[strProperty])
	
	-- MQTT Broker
	elseif strProperty == "MQTT Broker" then
		
		BROKER = {}
	
		local broker = Properties["MQTT Broker"]
		local topic  = Properties["Subscription Topic"]
		local qos    = Properties["Subscription QOS"]
	
		if broker == "" or topic == "" then
			u.console("Broker Or Topic Missing")
			return
		end
	
		-- Create Client
		BROKER = MQTT.instance({ broker = broker, subscriptions = topic, qos = qos, timeout = 5 }, function(topic, message)
			ProcessMessage(topic, message)
		end)
		
		u.console('Connecting To "' .. broker .. '"')
	
		-- Connect To Broker
		BROKER:connect(function(args)
			if BROKER.connected then
				u.console('Connected To "' .. broker .. '" And Subscribed To "' .. topic .. '"')
				C4:UpdateProperty("Status", "Online")
			else
				if args then
					u.console('Disconnected From "' .. broker .. '"')	
				else	
					u.console('Unable To Connect To "' .. broker .. '"')
				end
				C4:UpdateProperty("Status", "Offline")		
			end
		end)
		
	-- Subscription Properties
	elseif strProperty == "Subscription Topic" or strProperty == "Subscription QOS" then
		OnPropertyChanged("MQTT Broker")
	end
	
end

function ExecuteCommand(strCommand, tParams)
	
	u.debug("ExecuteCommand (" .. strCommand .. ")")
	
	if tParams then
		for i,v in pairs(tParams) do 
			if type(v) == "table" then
				u.debug(" Parameter: " .. i .. " = Table")
			else
				u.debug(" Parameter: " .. i .. " = " .. v)
			end
		end
	end
	
	-- Lua Window Action
	if (strCommand == "LUA_ACTION") then
		
		-- Publish Message
		if tParams.ACTION == "PUBLISH_MESSAGE" then
			
			local broker  = Properties["MQTT Broker"]
			local topic   = tParams["Topic"]
			local qos     = tParams["Quality Of Service"]
			local message = tParams["Message"]
			local retain  = tParams["Retain"] == "Yes"
			
			if broker == "" or topic == "" then
				u.console("Broker Or Topic Missing")
				return
			end
	
			BROKER:publish(topic, message, qos, retain, function(args)
				u.console('Published "' .. message .. '" To ' .. topic .. " With QOS " .. qos)
			end)
			
		end

	-- MQTT Message
	elseif strCommand == "MQTT" then
	
		if BROKER.handler then
			BROKER:handler(tParams)
		end
		
	end
	
end

-- On Driver Late Initialisation
function OnDriverLateInit()
	
	-- Initialise Properties
	OnPropertyChanged("MQTT Broker")
	OnPropertyChanged("Debug Mode")
	
end


-- Driver Functions
function ProcessMessage(topic, message)
	
	if not topic or not message then
		return
	end
	
	u.console('Received "' .. message .. '" On ' .. topic)
	
end